import math

import cocos
from pymunk.vec2d import Vec2d

import Actor
import Actor.Static
import Actor.Sticky
import Actor.Goal
import Actor.Spawner

class Level(cocos.cocosnode.CocosNode):
    def __init__(self, context):
        cocos.cocosnode.CocosNode.__init__(self)
        self.actors = []

    def update(self, context):
        for actor in self.actors:
            actor.update(context)

        for actor in [a for a in self.actors]:
            if actor.x < 0:
                self.removeActor(context, actor)
            elif actor.x > 1100:
                self.removeActor(context, actor)
            elif actor.y < 0:
                self.removeActor(context, actor)
            elif actor.y > 1000:
                self.removeActor(context, actor)


    def isWon(self):
        for actor in self.actors:
            if isinstance(actor, Actor.Goal.Goal):
                if actor.score != actor.target:
                    return False

        return True

    def isLost(self):

        for actor in self.actors:
            if isinstance(actor, Actor.Spawner.Spawner):
                if actor.ammo == 0:
                    return True

        return False


    def get_closest_actor(self, x, y):
        mindist = 100000000
        minactor= None
        for actor in self.actors:

            dist = (actor.x - x)**2 + (actor.y - y)**2
            if dist < mindist:
                mindist = dist
                minactor = actor

        return math.sqrt(mindist), minactor

    def get_closest_actor_by_type(self, x, y, actortype):
        mindist = 10000000
        minactor= None

        for actor in self.actors:
            if not isinstance(actor, actortype):
                #print "fail"
                continue
            dist = (actor.x - x)**2 + (actor.y - y)**2
            #print actor, dist
            if dist < mindist:
                mindist = dist
                minactor = actor

        return math.sqrt(mindist), minactor


    #### Factory ish functions
    def createActor(self, context, actortype, x, y, rot=0, data=""):
        a = None
        if actortype == "Static":
            a = Actor.Static.Static(context, Vec2d(x,y))
        if "Blob" in actortype:
            a = Actor.Blob.Blob(context, Vec2d(x,y), actortype)
        if "Sticky" in actortype:
            a = Actor.Sticky.Sticky(context, Vec2d(x,y))
        if "Goal" in actortype:
            a = Actor.Goal.Goal(context, Vec2d(x,y))
        if "Spawner" in actortype:
            a = Actor.Spawner.Spawner(context, Vec2d(x,y))
        if "Sticky" in actortype:
            a = Actor.Sticky.Sticky(context, Vec2d(x,y))

        if a == None:
            raise Exception("No such actor %s" % actortype)

        a.rotate(rot)
        self.add(a)
        self.actors.append(a)
        return a


    def removeActor(self, context, actor):
        self.remove(actor)
        self.actors.remove(actor)
        actor.destroy(context)


    def load(self, context, filename):
        print "Loading %s" % filename

        for a in self.actors:
            a.destroy(context)
            self.remove(a)

        self.actors = []

        f = open(filename)
        for line in f:
            if line == "" or line[0] == "#":
                continue

            tokens = line.strip().split()
            actortype, x, y, rot, data = tokens
            data = data.replace("___", " ")
            a = self.createActor(context, actortype, int(x), int(y), float(rot), data)
            a.load(context, data)
            print "...........................",actortype, data

        f.close()

    def save(self, context, filename):
        print "Saving %s" % filename

        f = open(filename, "w")
        actors = [a for a in self.actors if not isinstance(a, Actor.Blob.Blob)]

        #actors.sort()
        for a in actors:
            actortype = "?"
            if isinstance(a, Actor.Blob.Blob):
                actortype = "Blob"
                continue
            if isinstance(a, Actor.Static.Static):
                actortype = "Static"
            if isinstance(a, Actor.Goal.Goal):
                actortype = "Goal"
            if isinstance(a, Actor.Spawner.Spawner):
                actortype = "Spawner"
            if isinstance(a, Actor.Sticky.Sticky):
                actortype = "Sticky"

            data = a.save().replace(" ", "___")
            f.write("%s %d %d %.2f %s\n" % (actortype, a.x, a.y, a.rotation, data))

        f.close()
